
#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include <string.h>
#include "alg.h"
#include "header.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"


extern int SHA1RandomGenerateKey(char *byteBuffer, size_t byteBufferSize, char *seedBuffer, size_t seedBufferSize );



void bcattach(int argc, char ** argv)
{

    int fd,n;
    long Offset;
    ALG_SERV Alg;
    DWORD KeyHandle;
    alg_info *alg_inf;
    char pool[POOL_SIZE_BYTES];
    char *Key;
    char *password;


    if ( NULL == o_alg )
    {
        msg(stderr,"Please specify algorithm\n"); 
        usage(stdout);
        exit(-1);
    }
    if ( argc < 3 )
    {
        msg(stderr,"Please specify file name and device\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 3 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    if ( (Alg.bc_fd = open(argv[2], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[2],strerror(errno));
        exit(-1);
    }
    
    if ( 0 != bc_get_info(Alg,&n,&n) ) 
    {
        msg(stderr,"device %s busy\n",argv[2]);
        exit(-1);
    }

    if ( (fd = open(argv[1], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        exit(-1); 
    }
    
    if ( (alg_inf = GetInfoByName(o_alg)) == NULL )
    {
        msg(stderr,"Unknown algorithm : %s\n",o_alg);
        exit(-1);
    }

    Alg.alg_id=alg_inf->id;
    Alg.alg_module=alg_inf->mod_name;
    Offset=0;

    if (NULL == (password = getpass("Enter password:")))
    {
        msg(stderr,"Cannot read password\n");
        exit(-1);
    }
    
    Key = malloc(alg_inf->key_size/8);
    if (NULL == Key)
    {
        msg(stderr,"malloc(%d): %s\n",alg_inf->key_size/8,strerror(errno));
        exit(-1);
    }
    memset(pool,0,sizeof(pool));

    if (SHA1RandomGenerateKey(Key, alg_inf->key_size/8, password, strlen(password)))
    {
        msg(stderr,"Cannot generate Key\n");
        exit(-1);
    }
    
    Error=bc_set_fd(Alg, Key, alg_inf->key_size, pool, fd, Offset, 0);
    if ( Error!=0 )
    {
        print_error(Error);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }

    FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

}

