/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of create container routine
*
*******************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>
#include <string.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <sys/types.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mount.h>
#include <kblock.h>
#include "alg.h"
#include "header.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

#ifndef S_BLKSIZE
#define S_BLKSIZE	512	/* Block size for `st_blocks'.  */
#endif

#ifndef BLKGETSIZE
#include <linux/fs.h>
#endif

ALG_SERV Alg;
DWORD KeyHandle;
char *ContName=NULL;
DATA_BLOCK *pdb=NULL;


void CreateSignalHandler(int sig)
{

    if ( ask_y_n("Do you really want to terminate process? (y/n) [n]:")==FALSE )
        return;
    FreeKeyHandle(Alg,KeyHandle);
    if ( pdb!=NULL )FreeDataBlock((BYTE **)&pdb);
//    if ( ContName!=NULL ) unlink(ContName);   //Not really needed
    exit(-1);
}


void bccreate (int argc, char ** argv)
{

    struct HiddenSector  hs;
    int fd;
    DWORD DataSize;
    alg_info *alg_inf;
    unsigned long int i;
    time_t t,ot;
    int pr=0;
    int opr=0;
    struct stat st;
    long l;
    int is_dev=0;


    if ( argc < 2 )
    {
        fprintf(stderr,"Please specify container name\n"); 
        exit(-1); 
    }
    else if ( argc > 2 )
    {
        fprintf(stderr,"Too many parameters\n");
        exit(-1);
    }
    if ( o_alg==NULL )
    {
        fprintf(stderr,"Please specify algorithm name\n");
        exit(-1);
    }

    if (0 == access(argv[1],F_OK) &&
        -1!= lstat(argv[1],&st) && S_ISBLK(st.st_mode))
    {
        is_dev=1;
    }
    
    if ( o_cont_size==0 && !is_dev)
    {
        fprintf(stderr,"Please specify container size\n");
        exit(-1);
    }

    if ( 0 != o_cont_size && is_dev)
    {
        fprintf(stderr,"Ignoring \"-s\" parameter\n");
    }
    
    if (is_dev)
    {
        if (0 != access(argv[1],W_OK|R_OK))
        {
            perror(argv[1]);
            exit(-1);
        }
        if (!ask_y_n("Warning: All data on specified block device will be lost! Continue? (y/[n]):"))
        {
            exit(-1);
        }
    }
    
    if ( (alg_inf=GetInfoByName(o_alg))==NULL )
    {
        fprintf(stderr,"Unknown algorithm: %s\n",o_alg);
        exit(-1);
    }

    SetSignalHandlers(CreateSignalHandler);    

    
    if ( is_dev )
    {
        if ( (fd = open(argv[1], O_RDWR)) < 0 )
        {
            perror (argv[1]);
            exit(-1); 
        }
    }
    else
    {
        if ( (fd = open(argv[1], O_RDWR|O_CREAT|O_EXCL)) < 0 )
        {
            perror (argv[1]);
            exit(-1); 
        }
    }
    ContName=argv[1];

    if ( is_dev && !ioctl(fd,BLKGETSIZE,&l) )
        o_cont_size=(long long)l*S_BLKSIZE;

    
    /* setting defaults for hidden sector */
    memset(&hs,0,sizeof(hs));
    strncpy(hs.br.OEMid,"LOCOS94 ",8);
    strncpy(hs.br.volumeLabel,"CRYPTED_DSK",11);

    hs.br.jmpCode[0] = 0xEB;
    hs.br.jmpCode[1] = 0x3C;
    hs.br.jmpCode[2] = 0x90;

    hs.dwKeySize = sizeof(DATA_BLOCK);

    hs.dwDataOffset = ( 512 + sizeof(DATA_BLOCK) + 511 ) & ~511;

//    hs.br.bpb.totalSectorsLong = (DWORD)((o_cont_size - hs.dwDataOffset) / 512) ;
    hs.br.bpb.totalSectorsLong = (DWORD)((o_cont_size) / 512) ;
    hs.description[0]='\0';
    if ( o_description!=NULL )
    {
        strncpy(hs.description,o_description,65);
        hs.description[65]=0;
    }
    hs.algorithmId=alg_inf->id;
    hs.keyGenId=KGSHA_ID;   
    /* end of setting defaults */

    if ( (Alg.bc_fd = open("/dev/bcrypt0", O_RDONLY)) < 0 )
    {
        perror("/dev/bcrypt0");
        exit(-1);
    }
    
    Alg.alg_id=alg_inf->id;
    Alg.alg_module =alg_inf->mod_name;
    DataSize = sizeof(DATA_BLOCK);

    CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_CREATE_WITH_SINGLE_PASSWORD,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
    if ( Error!=0 )
    {
        print_error(Error);
        close(fd);
        if (!is_dev)
            unlink(ContName);
        exit(-1);
    }
    if ( write(fd,&hs,sizeof(hs)) < 0 )
    {
        perror (argv[1]);
        FreeKeyHandle(Alg,KeyHandle);
        FreeDataBlock((BYTE **)&pdb);
        close(fd);
        if (!is_dev)
            unlink(ContName);
        exit(-1); 
    }
    if ( write(fd,pdb,sizeof(DATA_BLOCK)) < 0 )
    {
        perror (argv[1]);
        FreeKeyHandle(Alg,KeyHandle);
        FreeDataBlock((BYTE **)&pdb);
        close(fd);
        if (!is_dev)
            unlink(ContName);
        exit(-1); 
    }
    
    
    if (is_dev)
    {
        FreeKeyHandle(Alg,KeyHandle);
        FreeDataBlock((BYTE **)&pdb);
        
        close(fd);
        close(Alg.bc_fd);
        return;
    }

    
    memset (pdb,0,sizeof(DATA_BLOCK));
    //o_cont_size-=sizeof(hs)+sizeof(DATA_BLOCK);
    ot=t=time(NULL);
    
    if ( o_verbose )
        fprintf(stdout,"Creating container %s\n",argv[1]);
    for ( i = o_cont_size+(hs.dwDataOffset-sizeof(hs)-sizeof(DATA_BLOCK)); i >= sizeof(DATA_BLOCK); i -= sizeof(DATA_BLOCK) )
    {
        if ( write(fd,pdb,sizeof(DATA_BLOCK)) < 0 )
        {
            perror (argv[1]);
            FreeKeyHandle(Alg,KeyHandle);
            FreeDataBlock((BYTE **)&pdb);
            close(fd);
            unlink(ContName);
            exit(-1); 
        }

        if ( o_verbose )
        {
            t=time(NULL);
            pr=100 - (int)((long long)i*100/o_cont_size);
            if ( t != ot )
            {
                fprintf(stdout,"%10ld/%ld Kb (%3d%%)            \r",(unsigned long int)((o_cont_size-i)>>10),(unsigned long int)(o_cont_size>>10),pr);
                ot=t;
                opr=pr;
            }
            else if ( pr != opr )
            {
                fprintf(stdout,"%10ld/%ld Kb (%3d%%)            \r",(unsigned long int)((o_cont_size-i)>>10),(unsigned long int)(o_cont_size>>10),pr);
                opr=pr;
            }
        }
    }
    
    if ( write(fd,pdb,i) < 0 )
    {
        perror (argv[1]);
        FreeKeyHandle(Alg,KeyHandle);
        FreeDataBlock((BYTE **)&pdb);
        close(fd);
        unlink(ContName);
        exit(-1); 
    }

    if ( o_verbose )
    {
        fprintf(stdout,"%10ld/%ld Kb (100%%)\n",(unsigned long int)(o_cont_size>>10),(unsigned long int)(o_cont_size>>10));
    }

    FreeKeyHandle(Alg,KeyHandle);
    FreeDataBlock((BYTE **)&pdb);

    fchmod(fd,S_IRUSR|S_IWUSR);

    close(fd);
    close(Alg.bc_fd);

}
