/*******************************************************************
*
*      Copyright (c) 1994-2000 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bclink.c
*      Revision:
*      Created:
*      Description: implementation of mount container routine
*
*******************************************************************/



#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include <string.h>
#include "alg.h"
#include "header.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"


extern int SHA1RandomGenerateKey(char *byteBuffer, size_t byteBufferSize, char *seedBuffer, size_t seedBufferSize );


void bclink_raw(int argc, char ** argv)
{

    int fd,n;
    long Offset;
    ALG_SERV Alg;
    DWORD KeyHandle;
    alg_info *alg_inf;
    char pool[POOL_SIZE_BYTES];
    char *Key;
    char *password;


    if ( NULL == o_alg )
    {
        msg(stderr,"Please specify algorithm\n"); 
        usage(stdout);
        exit(-1);
    }
    if ( argc < 3 )
    {
        msg(stderr,"Please specify file name and device\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 3 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    if ( (Alg.bc_fd = open(argv[2], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[2],strerror(errno));
        exit(-1);
    }
    
    if ( 0 != bc_get_info(Alg,&n,&n) ) 
    {
        msg(stderr,"device %s busy\n",argv[2]);
        exit(-1);
    }

    if ( (fd = open(argv[1], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        exit(-1); 
    }
    
    if ( (alg_inf = GetInfoByName(o_alg)) == NULL )
    {
        msg(stderr,"Unknown algorithm : %s\n",o_alg);
        exit(-1);
    }

    Alg.alg_id=alg_inf->id;
    Alg.alg_module=alg_inf->mod_name;
    Offset=0;

    if (NULL == (password = getpass("Enter password:")))
    {
        msg(stderr,"Cannot read password\n");
        exit(-1);
    }
    
    Key = malloc(alg_inf->key_size/8);
    if (NULL == Key)
    {
        msg(stderr,"malloc(%d): %s\n",alg_inf->key_size/8,strerror(errno));
        exit(-1);
    }
    memset(pool,0,sizeof(pool));

    if (SHA1RandomGenerateKey(Key, alg_inf->key_size/8, password, strlen(password)))
    {
        msg(stderr,"Cannot generate Key\n");
        exit(-1);
    }

    memset(password,0,strlen(password));
    
    Error=bc_set_fd(Alg, Key, alg_inf->key_size, pool, fd, Offset, 0);
    if ( Error!=0 )
    {
        print_error(Error);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }

    FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

}


void bclink (int argc, char ** argv)
{

    struct HiddenSector  hs;
    int fd,n;
    long Offset;
    ALG_SERV Alg;
    DWORD KeyHandle,DataSize;
    DATA_BLOCK db, *pdb;
    alg_info *alg_inf;


    if (o_raw_mode)
    {
        bclink_raw(argc,argv);
        return;
    }
    
    if ( argc < 3 )
    {
        msg(stderr,"Please specify container name and device\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 3 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    if ( (Alg.bc_fd = open(argv[2], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[2],strerror(errno));
        exit(-1);
    }
    
    if ( 0 != bc_get_info(Alg,&n,&n) ) 
    {
        msg(stderr,"device %s busy\n",argv[2]);
        exit(-1);
    }

    if ( (fd = open(argv[1], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        exit(-1); 
    }
    if ( read(fd, &hs, sizeof(hs))!=sizeof(hs) )
    {
        msg(stderr,"Can not read container header\n");
        exit(-1); 
    }
    if ( strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0 )
    {
        msg(stderr,"%s is not valid BestCrypt container\n",argv[1]);
        exit(-1);
    };   
    if ( read(fd, &db, sizeof (db))!=sizeof(db) )
    {
        msg(stderr,"Can not read data block\n");
        exit(-1); 
    }
    if ( (alg_inf=GetInfoById(hs.algorithmId))==NULL )
    {
        msg(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
        exit(-1);
    }

    Alg.alg_id=hs.algorithmId;
    Alg.alg_module =alg_inf->mod_name;
    pdb = &db;
    DataSize = sizeof(db);
    Offset=hs.dwDataOffset;

    CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
    if ( Error!=0 )
    {
        print_error(Error);
        exit(-1);
    }

    Error=bc_set_fd_kh(Alg, KeyHandle, fd, Offset, 0);
    if ( Error!=0 )
    {
        print_error(Error);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }

    FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

}



void bcunlink (int argc, char ** argv)
{

    ALG_SERV Alg;

    if ( argc < 2 )
    {
        msg(stderr,"Please specify device\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 2 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    if ( (Alg.bc_fd = open(argv[1], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        exit(-1);
    }

    Error=bc_clear_fd(Alg);
    if ( Error!=0 )
    {
        msg(stderr,"clear_fd(%s): ",argv[1]);
        print_error(Error);
        close(Alg.bc_fd);
        exit(-1);
    }

    close(Alg.bc_fd);

}

