/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of mount container routine
*
*******************************************************************/



#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <sys/mount.h>
#ifndef MS_MGC_VAL
    #include <linux/fs.h>
#endif
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include <string.h>
#include "alg.h"
#include "header.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

void bcmount (int argc, char ** argv)
{

    struct HiddenSector  hs;
    char *device;
    int fd;
    long Offset;
    ALG_SERV Alg;
    DWORD KeyHandle,DataSize;
    DATA_BLOCK db, *pdb;
    alg_info *alg_inf;

    FILE *mtab;
    struct mntent mnt;
    char rp1[NAME_MAX+PATH_MAX];
    char rp2[NAME_MAX+PATH_MAX];
    char options[256];
    unsigned long flags;
    int fs_id;



    if ( argc < 3 )
    {
        msg(stderr,"Please specify container name and mount point\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 3 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    
    Error=IsOwner(ruid,argv[2]);
    if ( Error==FALSE && ruid!=0 )
    {
        msg(stderr,"sorry, you are not owner of %s\n");
        exit(-1);
    }
    else if ( Error==-1 )
    {
        msg(stderr,"lstat(%s): %s\n",argv[2],strerror(errno));
        exit(-1);
    }

    if ( (device=FindFreeDevice())==NULL )
    {
        msg(stderr,"Can not find free device\n");
        exit(-1);
    }

    if ( (Alg.bc_fd = open(device, O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",device,strerror(errno));
        exit(-1);
    }
    if ( (fd = open(argv[1], O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s): %s\n",argv[1],strerror(errno));
        exit(-1); 
    }
    if ( read(fd, &hs, sizeof(hs))!=sizeof(hs) )
    {
        msg(stderr,"read(%s,&hs,%d): %s\n",argv[1],sizeof(hs),strerror(errno));
        exit(-1); 
    }
    if ( strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0 )
    {
        msg(stderr,"%s is not valid BestCrypt container\n",argv[1]);
        exit(-1);
    };   
    if ( read(fd, &db, sizeof (db))!=sizeof(db) )
    {
        msg(stderr,"read(%s,&db,%d): %s\n",argv[1],sizeof(hs),strerror(errno));
        exit(-1); 
    }
    if ( (alg_inf=GetInfoById(hs.algorithmId))==NULL )
    {
        msg(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
        exit(-1);
    }

    Alg.alg_id=hs.algorithmId;
    Alg.alg_module =alg_inf->mod_name;
    pdb = &db;
    DataSize = sizeof(db);
    Offset=hs.dwDataOffset;

    CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
    if ( Error!=0 )
    {
        print_error(Error);
        exit(-1);
    }

    Error=bc_set_fd_kh(Alg, KeyHandle, fd, Offset, 0);
    if ( Error!=0 )
    {
        print_error(Error);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }
    if ( o_fstype==NULL && (o_fstype=getFStypeById(hs.fileSystemId))==NULL )
    {
        msg(stderr,"Unknown filesystem. Try: -t fstype \n");
        bc_clear_fd(Alg);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }

    if ( setreuid(ruid,euid)!=0 )
    {
        msg(stderr,"setreuid(%d,%d): %s\n",ruid,euid,strerror(errno));
//        msg(stderr,"setreuid before mount:");
        print_error(errno); 
        bc_clear_fd(Alg);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }
                 
    if (0 != access(argv[1],R_OK|W_OK))
    {
        msg(stderr,"mounting read-only\n");
        o_ro=1;
    }

    snprintf(options,sizeof(options),"uid=%d,gid=%d,umask=%o",o_uid,o_gid,(~o_mode)&07777);
    flags=MS_MGC_VAL|(MS_RDONLY & o_ro);

    //
    // disable suid and dev files if not root 
    //
    if ( ruid!=0 )
        flags|=MS_NOSUID|MS_NODEV;

    fs_id=getIdByFStype(o_fstype);
    if ( fs_id == 1 || fs_id == 2 || fs_id == 3)
    {
        if ( mount(device,argv[2],o_fstype,flags,NULL)!=0 )
        {
            msg(stderr,"mount(%s,%s,%s,%X,NULL): %s\n",device,argv[2],o_fstype,flags,strerror(errno));
            bc_clear_fd(Alg);
            FreeKeyHandle(Alg,KeyHandle);
            exit(-1);
        }
    }
    else if ( mount(device,argv[2],o_fstype,flags,options)!=0 &&
              mount(device,argv[2],o_fstype,flags,NULL)!=0 )
    {
        msg(stderr,"mount(%s,%s,%s,%X,%s): %s\n",device,argv[2],o_fstype,flags,options,strerror(errno));
        bc_clear_fd(Alg);
        FreeKeyHandle(Alg,KeyHandle);
        exit(-1);
    }


    Error=bc_lock_dev(Alg);
    if ( Error!=0 )
    {
        print_error(Error);
        exit(-1);
    }


    if ( (mnt.mnt_fsname=realpath(argv[1],rp1)) == NULL ) 
        msg(stderr,"realpath(%s,rp1): %s\n",argv[1],strerror(errno));
    if ( (mnt.mnt_dir=realpath(argv[2],rp2)) == NULL )
        msg(stderr,"realpath(%s,rp2): %s\n",argv[2],strerror(errno));
    mnt.mnt_type=o_fstype;
    mnt.mnt_opts= o_ro ? "ro,BestCrypt" : "rw,BestCrypt";
    mnt.mnt_freq=0;
    mnt.mnt_passno=0;
    mtab = setmntent("/etc/mtab",  "r+");
    if ( mtab != NULL )
    {
        addmntent(mtab, &mnt);
        endmntent(mtab);
    }
    else msg(stderr,"Can not open /etc/mtab. Mount entry will not be added in mtab file.\n");


    FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

    chown(rp2,o_uid,o_gid);
    chmod(rp2,o_mode);

}

void bcumount (int argc, char ** argv)
{

    ALG_SERV Alg;
    char *device;
    FILE *mtab,*tmptab;
    struct mntent *pmnt;
    char rp[MAXNAMLEN];

    if ( argc < 2 )
    {
        msg(stderr,"Please specify mount point\n"); 
        usage(stdout);
        exit(-1);
    }
    else if ( argc > 2 )
    {
        msg(stderr,"Too many parameters\n");
        usage(stdout);
        exit(-1);
    }

    if ( NULL == realpath(argv[1],rp) )
    {
        msg(stderr,"realpath(%s,rp): %s\n",argv[1],strerror(errno));
        exit(-1);
    };

    Error=IsOwner(ruid,rp);
    if ( Error==FALSE && ruid!=0 )
    {
        msg(stderr,"sorry, you are not owner of this mount point");
        exit(-1);
    }
    else if ( Error==-1 )
    {
        msg(stderr,"IsOwner(%d,rp): %s\n",ruid,strerror(errno));
//        print_error(errno);
        exit(-1);
    }
    
    sync();

    device=FindMountedDevice(rp);
    if ( device==NULL )
    {
        msg(stderr,"Can not find mounted device\n");
        exit(-1);
    }

    if ( (Alg.bc_fd = open(device, O_RDONLY)) < 0 )
    {
        msg(stderr,"open(%s,O_RDONLY): %s\n",device,strerror(errno));
        exit(-1);
    }

    if ( setreuid(euid,euid)!=0 )
    {
        msg(stderr,"setreuid(%d,%d): %s\n",euid,euid,strerror(errno));
        close(Alg.bc_fd);
        exit(-1);
    }
    
    Error=bc_unlock_dev(Alg);
    if ( Error!=0 )
    {
        print_error(Error);
        close(Alg.bc_fd);
        exit(-1);
    }

    if ( umount(device)!=0 )
    {
        msg(stderr,"umouns(%s): %s\n",device,strerror(errno));
        close(Alg.bc_fd);
        exit(-1);
    }

    mtab   = setmntent("/etc/mtab",  "r");
    tmptab = setmntent("/etc/mtab.tmp", "w+");

    while ( 1 )
    {
        pmnt=getmntent(mtab);
        if ( feof(mtab) )break;
        if ( strcmp(rp,pmnt->mnt_dir) )
            addmntent(tmptab, pmnt);
    }

    endmntent(mtab);
    endmntent(tmptab);

    if ( rename("/etc/mtab","/etc/mtab~")!=0 )
        msg(stderr,"rename(\"/etc/mtab\",\"/etc/mtab~\"): %s\n",strerror(errno));
    if ( rename("/etc/mtab.tmp","/etc/mtab")!=0 ) 
        msg(stderr,"rename(\"/etc/mtab.tmp\",\"/etc/mtab\"): %s\n",strerror(errno));
    if ( chmod("/etc/mtab",0644)!=0 ) 
        msg(stderr,"chmod(\"/etc/mtab\",0644): %s\n",strerror(errno));
    if ( unlink("/etc/mtab~")!=0 ) 
        msg(stderr,"unlink(\"/etc/mtab~\"): %s\n",strerror(errno));

    Error=bc_clear_fd(Alg);
    if ( Error!=0 )
    {
        print_error(Error);
        close(Alg.bc_fd);
        exit(-1);
    }

    close(Alg.bc_fd);

}



