/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of command line interface
*
*******************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <bc_types.h>
#include "alg.h"
#include "config.h"
#include "error.h"
#include "version.h"
#include "bctool.h"
#include "misc.h"


extern void bccreate(int argc,char **argv);
extern void bcumount(int argc,char **argv);
extern void bcmount(int argc,char **argv);
extern void bcpasswd(int argc,char **argv);
extern void bcinfo(int argc,char **argv);
extern void bcformat(int argc,char **argv);
extern void bcreencrypt(int argc,char **argv);
extern void bcfsck(int argc,char **argv);
extern void bclink(int argc,char **argv);
extern void bcunlink(int argc,char **argv);

int main (int argc, char ** argv)
{
    char c;

    Error=0;
    o_fstype=NULL;
    o_alg=NULL;
    o_options=NULL;
    o_cont_size=0;
    o_description=NULL;
    o_verbose=0;
    o_ro=0;
    o_mode=0700;


    setvbuf(stderr,(char*)NULL,_IONBF,0);
    setvbuf(stdout,(char*)NULL,_IONBF,0);


    ruid=getuid();
    euid=geteuid();
    if ( setreuid(euid,ruid)!=0 )
    {
        perror("setreuid:"); exit(-1);
    }
    o_uid=ruid;
    o_gid=getgid();

    while ( (c=getopt(argc,argv,"VvhrRt:a:o:s:d:m:u:g:"))!=EOF )
    {
        switch ( c )
        {
            case 'V':
                printf("BestCrypt version %s\n",VERSION);
                exit(-1);
            case 'v':
                o_verbose++;
                break;
            case 'r':
                o_ro=0xff;
                break;          
            case 'R':
                o_raw_mode=0xff;
                break;          
            case 'h':
                usage(stdout);
                exit(-1);
            case 't':
                o_fstype=optarg;
                break;
            case 'a':
                o_alg=strtolower(optarg);
                break;
            case 'o':
                o_options=optarg;
                break;
            case 'd':
                o_description=optarg;
                break;
            case 's':
                o_cont_size=string_to_long(optarg);
                if ( o_cont_size==0 )
                {
                    msg(stderr,"Invalid container size\n");
                    exit(-1);
                }
                break;
            case 'm':
                o_mode=strtol(optarg,NULL,8);
                o_mode=o_mode & 07777;
                if (0 == o_mode)
                {
                    msg(stderr,"Invalid mode. Using 0700\n");
                    o_mode=0700;
                }
                break;
            case 'u':
                if (0 != ruid) 
                {
                    msg(stderr,"Only root can specify mountpoint's owner\n");
                    exit(-1);
                }
                o_uid=get_uid_by_name(optarg);
                if (-1 == o_uid)
                {
                    msg(stderr,"Invalid username '%s'\n",optarg);
                    exit(-1);
                }
                break;
            case 'g':
                o_gid=get_gid_by_name(optarg);
                if (-1 == o_gid)
                {
                    msg(stderr,"Invalid groupname '%s'\n",optarg);
                    exit(-1);
                }
                break;
            case '?':
            default:
                usage(stderr);
                exit(-1);
        } /* end of switch */



    } /* end of while */

    argc -= optind;
    argv += optind;

    if ( argc==0 )
    {
        msg(stderr,"Missed command\n");
        usage(stderr);
        exit(-1);
    }

    if ( (Error=read_conf())!=0 )
    {
        print_error(Error); exit(-1);
    }

    if ( !modules_loaded() )
    {
        msg(stderr,"kernel modules not loaded\n");
        exit(-1);
    }

    if ( !strcmp(argv[0],"info") ) bcinfo(argc,argv);
    else if ( !strcmp(argv[0],"new") ) bccreate(argc,argv);
    else if ( !strcmp(argv[0],"format") ) bcformat(argc,argv);
    else if ( !strcmp(argv[0],"mount") ) bcmount(argc,argv);
    else if ( !strcmp(argv[0],"umount") ) bcumount(argc,argv);
    else if ( !strcmp(argv[0],"passwd") )  bcpasswd(argc,argv);
    else if ( !strcmp(argv[0],"reencrypt") )  bcreencrypt(argc,argv);
    else if ( !strcmp(argv[0],"fsck") )  bcfsck(argc,argv);
    else if ( !strcmp(argv[0],"link") )  bclink(argc,argv);
    else if ( !strcmp(argv[0],"unlink") )  bcunlink(argc,argv);
    else
    {
        msg(stderr,"Unknown command %s\n",argv[0]);
        usage(stderr);
        exit(-1);
    }

    exit(0);
} /* end of main */

