/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of miscelleneous routines
*
*******************************************************************/



#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include "header.h"
#include "alg.h"
#include "config.h"
#include "error.h"
#include "fstypes.h" 


#define MAX_DEV_PATH 16
#define MAX_DEVICES 16


char _device[MAX_DEV_PATH];

void SetSignalHandlers(void (*handler)(int))
{

    signal(SIGINT,handler);
    signal(SIGQUIT,handler);
    signal(SIGTERM,handler);
    signal(SIGABRT,handler);

}



char* strtolower(char *string)
{
    int i;
    for ( i=0;i<=strlen(string);i++ )string[i]=tolower(string[i]);
    return string;

}


void msg(FILE *fd, const char *format, ... )
{

    va_list args;

    va_start(args, format);
    vfprintf(fd, format, args);
    va_end(args);                                                                                                   

}

int is_valid_container(char *filename)
{

    struct HiddenSector  hs;
    int fd;


    if ( (fd = open(filename, O_RDONLY)) < 0 )
    {
        perror (filename);
        return FALSE;
    }
    if ( read(fd, &hs, sizeof(hs))!=sizeof(hs) )
    {
        msg(stderr,"Can not read container header\n");
        close(fd);
        return FALSE;
    }

    close(fd);

    if ( strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0 )
    {
        return FALSE;
    }

    if ( strncmp(hs.br.volumeLabel, "CRYPTED_DSK", sizeof(hs.br.volumeLabel))!=0 )
    {
        return FALSE;
    }

    return TRUE;

};




long long int string_to_long(char *string)
{

    char *endptr;
    unsigned long int result;

    result=strtoul(string,&endptr,0);
    if ( string==endptr ) return 0;
    if ( (string[0]!='\0')&&(endptr[0]=='\0') ) return result;
    if ( (endptr[0]=='K' || endptr[0]=='k' )&&(endptr[1]=='\0') )
    {
        result *=1024;
        return result;
    };
    if ( (endptr[0]=='M' || endptr[0]=='m')&&(endptr[1]=='\0') )
    {
        result *=1024*1024;
        return result;
    };
    return 0;

}


char* FindFreeDevice()
{

    int i,n,rc;
    ALG_SERV Alg;

    for ( i=0;i<MAX_DEVICES;i++ )
    {
        snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt%d",i);
        Alg.bc_fd = open(_device, O_RDONLY);
        if ( Alg.bc_fd >= 0 )
        {
            rc=bc_get_info(Alg,&n,&n);
            close(Alg.bc_fd);
            if ( rc==0 ) return _device;
        }
    }

    return NULL;
}


char* FindMountedDevice(char *arg)
{

    struct stat buff1;
    int i,dev;

    if ( lstat(arg,&buff1)!=0 ) return NULL;
    if ( S_ISBLK(buff1.st_mode) ) return arg;
    dev=buff1.st_dev;

    for ( i=0;i<MAX_DEVICES;i++ )
    {
        snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt%d",i);
        if ( lstat(_device,&buff1)==0 && dev==buff1.st_rdev ) return _device;
    }

    return NULL;
}


int IsOwner(int uid, char *arg)
{

    struct stat buff;

    if ( lstat(arg,&buff)!=0 ) return -1;
    if ( buff.st_uid==uid ) return TRUE;
    return FALSE;

}   

/*
int run_process(char *file, char **argv)
{

    int pid,status;

    if ( file==NULL )return 1;
    pid=fork();
    if ( pid==-1 ) return -1;
    if ( pid==0 )
    {
        execve(file,argv,NULL);
        exit(127);
    }
    do
    {
        if ( waitpid(pid,&status,0)==-1 )
        {
            if ( errno != EINTR ) return -1;
        }
        else return status;
    } while ( 1 );

}

*/

/* Forks and execs the given command. The program will
   be run with empty environment to avoid LD_LIBRARY_PATH and similar
   attacks. */
int run_proc(const char *command)
{
    int pid, i, j;
    char *args[16];
    char *env[16];
    extern char **environ;

    pid = fork();
    if ( pid < 0 )
    {
        msg(stderr,"fork: %s", strerror(errno));
        return -1;
    }

    if ( pid == 0 )
    { /* Child */

        /* Build argument vector. */
        i = 0;
        args[i++] = "/bin/sh";
        args[i++] = "-c";
        args[i++] = (char *)command;
        args[i++] = NULL;

        /* Prune environment to remove any potentially dangerous variables. */
        i = 0;
        for ( j = 0; environ[j] && i < sizeof(env)/sizeof(env[0]) - 1; j++ )
            if ( strncmp(environ[j], "HOME=", 5) == 0 ||
                 strncmp(environ[j], "USER=", 5) == 0 ||
                 strncmp(environ[j], "PATH=", 5) == 0 ||
                 strncmp(environ[j], "LOGNAME=", 8) == 0 ||
                 strncmp(environ[j], "TZ=", 3) == 0 ||
                 strncmp(environ[j], "MAIL=", 5) == 0 ||
                 strncmp(environ[j], "SHELL=", 6) == 0 ||
                 strncmp(environ[j], "TERM=", 5) == 0 ||
                 strncmp(environ[j], "DISPLAY=", 8) == 0 ||
                 strncmp(environ[j], "PRINTER=", 8) == 0 ||
                 strncmp(environ[j], "XAUTHORITY=", 11) == 0 ||
                 strncmp(environ[j], "TERMCAP=", 8) == 0 )
                env[i++] = environ[j];
        env[i] = NULL;

        execve("/bin/sh", args, env);
        msg(stderr,"execv /bin/sh failed: %s", strerror(errno));
    }

    /* Parent. */
    wait(NULL);
    return 0;
}






//ask_y_n(char *string)
//asks for yes or no
//return value:
//TRUE if 'y' entered
//FALSE in other cases
int ask_y_n(char *string)
{
    char c;
    fprintf(stderr,string);
    c = tolower(fgetc(stdin));
    if ( '\n' == c ) return FALSE;
    while ( '\n' != fgetc(stdin) );
    if ( 'y' == c ) return TRUE;
    return FALSE;
}

int get_uid_by_name(const char *name)
{
    struct passwd *pass;
    if (NULL == name)
        return -1;
    pass=getpwnam(name);
    if (NULL == pass)
        return -1;
    return pass->pw_uid;
}
int get_gid_by_name(const char *name)
{
    struct group *grp;
    if (NULL == name)
        return -1;
    grp=getgrnam(name);
    if (NULL == grp)
        return -1;
    return grp->gr_gid;

}


char* getFStypeById(unsigned int id)
{

    if ( id > MAX_ID )return NULL;
    return fstypes[id];
}

int   getIdByFStype(char *fstype)
{

    int i;

    for ( i=0;i<MAX_ID;i++ )if ( strcmp(fstype,fstypes[i])==0 ) return i;
    return MAX_ID;

}

int modules_loaded()
{
    int n;
    ALG_SERV Alg;

    Alg.bc_fd = open("/dev/bcrypt0", O_RDONLY);
    
    if ( ENODEV == errno || -1 == bc_get_info(Alg,&n,&n))
    {
        close(Alg.bc_fd);
        return FALSE;
    }
    close(Alg.bc_fd);
    return TRUE;
}



void usage(FILE *fd)
{
    char *usage_string="Usage:\n\
bctool command [options] arguments\n\
\n\
commands:\n\
   new    - create container   (-a alg_name -s size [-d description] filename)\n\
   format - format container   ([-t fstype] filename)\n\
   mount  - mount container    ([-t fstype] [-m mode] [-u user] [-g group] [-r] filename mountpoint)\n\
   umount - unmount container  (mountpoint)\n\
   info   - get info on container (filename)\n\
   passwd - change container's password (filename)\n\
   fsck   - filesystem check (filename [-t fstype])\n\
   reencrypt - reencrypt container (-a alg_name filename)\n\
\n\
options:\n\
   -t fstype   - to specify filesystem type (default: msdos)\n\
   -a alg_name - to specify algorithm name\n\
   -s size     - to specify container size\n\
        example: -s xxx - in bytes; -s xxxK - in Kb; -s xxxM - in Mb\n\
   -d desc     - to specify description of container (up to 64 symbols)\n\
   -r          - to mount read only\n\
   -m mode     - to specify mountpoint mode in octal (default 0700)\n\
   -u user     - to specify mountpoint's owner (root only can do it)\n\
   -g group    - to specify mountpoint's group\n\
\n";

    fprintf(fd,usage_string);

}




